@@ -627,6 +627,25 @@ gupnp_device_info_get_upc (GUPnPDeviceInfo *info)
                                                         "UPC");
 }
 
+/**
+ * gupnp_device_info_get_presentation_url
+ * @info: A #GUPnPDeviceInfo
+ *
+ * Get a URL pointing to the device's presentation page, for web-based
+ * administration.
+ * 
+ * Return value: A string, or %NULL. g_free() after use.
+ **/
+char *
+gupnp_device_info_get_presentation_url (GUPnPDeviceInfo *info)
+{
+        g_return_val_if_fail (GUPNP_IS_DEVICE_INFO (info), NULL);
+
+        return xml_util_get_child_element_content_url (info->priv->element,
+                                                       "presentationURL",
+                                                       info->priv->url_base);
+}
+
 typedef struct {
         xmlChar *mime_type;
         int      width;
@@ -673,17 +692,17 @@ icon_free (Icon *icon)
 /**
  * gupnp_device_info_get_icon_url
  * @info: A #GUPnPDeviceInfo
- * @requested_mime_type: The requested file format, or NULL for any
+ * @requested_mime_type: The requested file format, or %NULL for any
  * @requested_depth: The requested color depth, or -1 for any
  * @requested_width: The requested width, or -1 for any
  * @requested_height: The requested height, or -1 for any
- * @prefer_bigger: TRUE if a bigger, rather than a smaller icon should be
+ * @prefer_bigger: %TRUE if a bigger, rather than a smaller icon should be
  * returned if no exact match could be found
  * @mime_type: The location where to store the the format of the returned icon,
- * or NULL. The returned string should be freed after use
- * @depth: The location where to store the depth of the returned icon, or NULL
- * @width: The location where to store the width of the returned icon, or NULL
- * @height: The location where to store the height of the returned icon, or NULL
+ * or %NULL. The returned string should be freed after use
+ * @depth: The location where to store the depth of the returned icon, or %NULL
+ * @width: The location where to store the width of the returned icon, or %NULL
+ * @height: The location where to store the height of the returned icon, or %NULL
  *
  * Get a URL pointing to the icon most closely matching the
  * given criteria, or %NULL. If @requested_mime_type is set, only icons with
@@ -772,6 +791,9 @@ gupnp_device_info_get_icon_url (GUPnPDeviceInfo *info,
                 }
         }
 
+        if (icons == NULL)
+                return NULL;
+
         /* Find closest match */
         closest = NULL;
         for (l = icons; l; l = l->next) {
