@@ -55,7 +55,7 @@ struct _GUPnPServiceProxyPrivate {
         char *path; /* Path to this proxy */
 
         char *sid; /* Subscription ID */
-        guint subscription_timeout_id;
+        GSource *subscription_timeout_src;
 
         int seq; /* Event sequence number */
 
@@ -63,8 +63,8 @@ struct _GUPnPServiceProxyPrivate {
 
         GList *pending_messages; /* Pending SoupMessages from this proxy */
 
-        GList *pending_notifies; /* Pending notifications to be sent (xmlDoc*) */
-        guint notify_idle_id; /* Idle handler ID of notification emiter */
+        GList *pending_notifies; /* Pending notifications to be sent (xmlDoc) */
+        GSource *notify_idle_src; /* Idle handler src of notification emiter */
 };
 
 enum {
@@ -227,7 +227,7 @@ gupnp_service_proxy_dispose (GObject *object)
         /* Cancel pending messages */
         context = gupnp_service_info_get_context (GUPNP_SERVICE_INFO (proxy));
         if (context)
-                session = _gupnp_context_get_session (context);
+                session = gupnp_context_get_session (context);
         else
                 session = NULL; /* Not the first time dispose is called. */
 
@@ -246,15 +246,16 @@ gupnp_service_proxy_dispose (GObject *object)
         }
 
         /* Cancel pending notifications */
-        if (proxy->priv->notify_idle_id) {
-                g_source_remove (proxy->priv->notify_idle_id);
-                proxy->priv->notify_idle_id = 0;
+        if (proxy->priv->notify_idle_src) {
+                g_source_destroy (proxy->priv->notify_idle_src);
+                proxy->priv->notify_idle_src = NULL;
         }
         
         while (proxy->priv->pending_notifies) {
                 xmlFreeDoc (proxy->priv->pending_notifies->data);
-                proxy->priv->pending_notifies = g_list_delete_link (proxy->priv->pending_notifies,
-                                                                   proxy->priv->pending_notifies);
+                proxy->priv->pending_notifies =
+                        g_list_delete_link (proxy->priv->pending_notifies,
+                                            proxy->priv->pending_notifies);
         }
         
         /* Call super */
@@ -339,8 +340,8 @@ gupnp_service_proxy_class_init (GUPnPServiceProxyClass *klass)
  * @proxy: A #GUPnPServiceProxy
  * @action: An action
  * @error: The location where to store any error, or %NULL
- * @Varargs: tuples of in parameter name, in paramater type, and in parameter
- * value, followed by %NULL, and then tuples of out paramater name,
+ * @Varargs: tuples of in parameter name, in parameter type, and in parameter
+ * value, followed by %NULL, and then tuples of out parameter name,
  * out parameter type, and out parameter value location, terminated with %NULL
  *
  * Sends action @action with parameters @Varargs to the service exposed by
@@ -381,8 +382,8 @@ stop_main_loop (GUPnPServiceProxy       *proxy,
  * @proxy: A #GUPnPServiceProxy
  * @action: An action
  * @error: The location where to store any error, or %NULL
- * @var_args: va_list of tuples of in parameter name, in paramater type, and in
- * parameter value, followed by %NULL, and then tuples of out paramater name,
+ * @var_args: va_list of tuples of in parameter name, in parameter type, and in
+ * parameter value, followed by %NULL, and then tuples of out parameter name,
  * out parameter type, and out parameter value location
  *
  * See gupnp_service_proxy_send_action(); this version takes a va_list for
@@ -495,7 +496,7 @@ gupnp_service_proxy_send_action_hash (GUPnPServiceProxy *proxy,
  * @callback: The callback to call when sending the action has succeeded
  * or failed
  * @user_data: User data for @callback
- * @Varargs: tuples of in parameter name, in paramater type, and in parameter
+ * @Varargs: tuples of in parameter name, in parameter type, and in parameter
  * value, terminated with %NULL
  *
  * Sends action @action with parameters @Varargs to the service exposed by
@@ -676,7 +677,7 @@ finish_action_msg (GUPnPServiceProxyAction *action,
                          "</s:Envelope>");
 
         soup_message_set_request (action->msg,
-                                  "text/xml",
+                                  "text/xml; charset=utf-8",
                                   SOUP_MEMORY_TAKE,
                                   action->msg_str->str,
                                   action->msg_str->len);
@@ -690,7 +691,7 @@ finish_action_msg (GUPnPServiceProxyAction *action,
         /* Send the message */
         context = gupnp_service_info_get_context
                                 (GUPNP_SERVICE_INFO (action->proxy));
-        session = _gupnp_context_get_session (context);
+        session = gupnp_context_get_session (context);
 
         soup_session_queue_message (session,
                                     action->msg,
@@ -717,7 +718,7 @@ write_in_parameter (const char *arg_name,
  * @callback: The callback to call when sending the action has succeeded
  * or failed
  * @user_data: User data for @callback
- * @var_args: A va_list of tuples of in parameter name, in paramater type, and
+ * @var_args: A va_list of tuples of in parameter name, in parameter type, and
  * in parameter value
  *
  * See gupnp_service_proxy_begin_action(); this version takes a va_list for
@@ -843,7 +844,7 @@ gupnp_service_proxy_begin_action_hash
  * @proxy: A #GUPnPServiceProxy
  * @action: A #GUPnPServiceProxyAction handle
  * @error: The location where to store any error, or %NULL
- * @Varargs: tuples of out parameter name, out paramater type, and out parameter
+ * @Varargs: tuples of out parameter name, out parameter type, and out parameter
  * value location, terminated with %NULL. The out parameter values should be
  * freed after use
  *
@@ -889,14 +890,14 @@ check_action_response (GUPnPServiceProxy       *proxy,
         case SOUP_STATUS_INTERNAL_SERVER_ERROR:
                 break;
         default:
-                set_server_error (error, action->msg);
+                _gupnp_error_set_server_error (error, action->msg);
 
                 return NULL;
         }
 
         /* Parse response */
-        response = xmlParseMemory (action->msg->response_body->data,
-                                   action->msg->response_body->length);
+        response = xmlRecoverMemory (action->msg->response_body->data,
+                                     action->msg->response_body->length);
 
 	if (!response) {
                 if (action->msg->status_code == SOUP_STATUS_OK) {
@@ -905,7 +906,7 @@ check_action_response (GUPnPServiceProxy       *proxy,
                                      GUPNP_SERVER_ERROR_INVALID_RESPONSE,
                                      "Could not parse SOAP response");
                 } else {
-                        set_error_literal
+                        g_set_error_literal
                                     (error,
                                      GUPNP_SERVER_ERROR,
                                      GUPNP_SERVER_ERROR_INTERNAL_SERVER_ERROR,
@@ -986,10 +987,10 @@ check_action_response (GUPnPServiceProxy       *proxy,
                 if (desc == NULL)
                         desc = g_strdup (action->msg->reason_phrase);
 
-                set_error_literal (error,
-                                   GUPNP_CONTROL_ERROR,
-                                   code,
-                                   desc);
+                g_set_error_literal (error,
+                                     GUPNP_CONTROL_ERROR,
+                                     code,
+                                     desc);
 
                 g_free (desc);
 
@@ -1010,7 +1011,7 @@ read_out_parameter (const char *arg_name,
 {
         xmlNode *param;
 
-        /* Try to find a matching paramater in the response*/
+        /* Try to find a matching parameter in the response*/
         param = xml_util_get_element (params,
                                       arg_name,
                                       NULL);
@@ -1029,7 +1030,7 @@ read_out_parameter (const char *arg_name,
  * @proxy: A #GUPnPServiceProxy
  * @action: A #GUPnPServiceProxyAction handle
  * @error: The location where to store any error, or %NULL
- * @var_args: A va_list of tuples of out parameter name, out paramater type,
+ * @var_args: A va_list of tuples of out parameter name, out parameter type,
  * and out parameter value location. The out parameter values should be
  * freed after use
  *
@@ -1180,7 +1181,7 @@ gupnp_service_proxy_cancel_action (GUPnPServiceProxy       *proxy,
         if (action->msg != NULL) {
                 context = gupnp_service_info_get_context
                                         (GUPNP_SERVICE_INFO (proxy));
-                session = _gupnp_context_get_session (context);
+                session = gupnp_context_get_session (context);
 
                 soup_session_cancel_message (session,
                                              action->msg,
@@ -1395,7 +1396,7 @@ emit_notifications (gpointer user_data)
                                             proxy->priv->pending_notifies);
         }
         
-        proxy->priv->notify_idle_id = 0;
+        proxy->priv->notify_idle_src = NULL;
 
 	return FALSE;
 }
@@ -1505,8 +1506,8 @@ server_handler (SoupServer        *soup_server,
                 proxy->priv->seq = 1;
 
         /* Parse the actual XML message content */
-        doc = xmlParseMemory (msg->request_body->data,
-                              msg->request_body->length);
+        doc = xmlRecoverMemory (msg->request_body->data,
+                                msg->request_body->length);
         if (doc == NULL) {
                 /* Failed */
                 g_warning ("Failed to parse NOTIFY message body");
@@ -1535,9 +1536,23 @@ server_handler (SoupServer        *soup_server,
 	 */
         proxy->priv->pending_notifies =
                 g_list_append (proxy->priv->pending_notifies, doc);
-        if (!proxy->priv->notify_idle_id)
-                proxy->priv->notify_idle_id =
-                        g_idle_add (emit_notifications, proxy);
+        if (!proxy->priv->notify_idle_src) {
+	        GUPnPContext *context;
+	        GMainContext *main_context;
+
+	        context = gupnp_service_info_get_context
+                        (GUPNP_SERVICE_INFO (proxy));
+	        main_context = gssdp_client_get_main_context
+                        (GSSDP_CLIENT (context));
+
+	        proxy->priv->notify_idle_src = g_idle_source_new();
+	        g_source_set_callback (proxy->priv->notify_idle_src,
+				       emit_notifications,
+                                       proxy, NULL);
+	        g_source_attach (proxy->priv->notify_idle_src, main_context);
+
+                g_source_unref (proxy->priv->notify_idle_src);
+	}
         
         /* Everything went OK */
         soup_message_set_status (msg, SOUP_STATUS_OK);
@@ -1574,7 +1589,7 @@ subscription_expire (gpointer user_data)
         proxy = GUPNP_SERVICE_PROXY (user_data);
 
         /* Reset timeout ID */
-        proxy->priv->subscription_timeout_id = 0;
+        proxy->priv->subscription_timeout_src = NULL;
 
         /* Send renewal message */
         context = gupnp_service_info_get_context (GUPNP_SERVICE_INFO (proxy));
@@ -1604,7 +1619,7 @@ subscription_expire (gpointer user_data)
         proxy->priv->pending_messages =
                 g_list_prepend (proxy->priv->pending_messages, msg);
 
-        session = _gupnp_context_get_session (context);
+        session = gupnp_context_get_session (context);
 
         soup_session_queue_message (session,
                                     msg,
@@ -1670,12 +1685,20 @@ subscribe_got_response (SoupSession       *session,
                 }
 
                 if (strncmp (hdr, "Second-", strlen ("Second-")) == 0) {
+                        GUPnPContext *context;
+                        GMainContext *main_context;
+
+                        context = gupnp_service_info_get_context
+                                (GUPNP_SERVICE_INFO (proxy));
+                        main_context = gssdp_client_get_main_context
+                                (GSSDP_CLIENT (context));
+
                         /* We have a finite timeout */
                         timeout = atoi (hdr + strlen ("Second-"));
 
                         /* We want to resubscribe before the subscription
                          * expires. */
-                        timeout -= GENA_TIMEOUT_DELTA;
+                        timeout = g_random_int_range (1, timeout / 2);
 
                         if (timeout < 0) {
                                 g_warning ("Invalid time-out specified. "
@@ -1686,19 +1709,26 @@ subscribe_got_response (SoupSession       *session,
                         }
 
                         /* Add actual timeout */
-                        proxy->priv->subscription_timeout_id =
-                                g_timeout_add_seconds (timeout,
-                                                       subscription_expire,
-                                                       proxy);
+                        proxy->priv->subscription_timeout_src =
+			        g_timeout_source_new_seconds (timeout);
+			g_source_set_callback
+                                (proxy->priv->subscription_timeout_src,
+                                 subscription_expire,
+                                 proxy, NULL);
+			g_source_attach (proxy->priv->subscription_timeout_src,
+					 main_context);
+
+                        g_source_unref (proxy->priv->subscription_timeout_src);
                 }
         } else {
                 GUPnPContext *context;
                 SoupServer *server;
 
                 /* Subscription failed. */
-                error = g_error_new (GUPNP_EVENTING_ERROR,
-                                     GUPNP_EVENTING_ERROR_SUBSCRIPTION_FAILED,
-                                     msg->reason_phrase);
+                error = g_error_new_literal
+                                (GUPNP_EVENTING_ERROR,
+                                 GUPNP_EVENTING_ERROR_SUBSCRIPTION_FAILED,
+                                 msg->reason_phrase);
 
  ERROR:
                 proxy->priv->subscribed = FALSE;
@@ -1804,7 +1834,7 @@ subscribe (GUPnPServiceProxy *proxy)
         proxy->priv->pending_messages =
                 g_list_prepend (proxy->priv->pending_messages, msg);
 
-        session = _gupnp_context_get_session (context);
+        session = gupnp_context_get_session (context);
 
         soup_session_queue_message (session,
                                     msg,
@@ -1846,7 +1876,7 @@ unsubscribe (GUPnPServiceProxy *proxy)
                                      proxy->priv->sid);
 
         /* And queue it */
-        session = _gupnp_context_get_session (context);
+        session = gupnp_context_get_session (context);
 
         soup_session_queue_message (session, msg, NULL, NULL);
 
@@ -1858,9 +1888,9 @@ unsubscribe (GUPnPServiceProxy *proxy)
         proxy->priv->seq = 0;
 
         /* Remove subscription timeout */
-        if (proxy->priv->subscription_timeout_id) {
-                g_source_remove (proxy->priv->subscription_timeout_id);
-                proxy->priv->subscription_timeout_id = 0;
+        if (proxy->priv->subscription_timeout_src) {
+                g_source_destroy (proxy->priv->subscription_timeout_src);
+                proxy->priv->subscription_timeout_src = NULL;
         }
 
         /* Remove server handler */
